#!/usr/bin/env startpython
from __future__ import print_function
#########################################################################
#                                                                       # 
#         PLAMS TEMPLATE FOR READING FROM KF-FILES                      # 
#                                                                       # 
#     More information:                                                 # 
#     PLAMS - www.scm.com/doc/Scripting/PLAMS/PLAMS.html                #
#     KF    - www.scm.com/doc/ADF/Appendices/Binary_result_files.html   #
#                                                                       #
#########################################################################


import sys
import os
import glob
import shutil
from scm.plams import *
from rdkit import Chem

#
# print help and exit
#
def help_n_exit(message):
    print message
    sys.exit(1)

#
# clean atom names 
#
def clean_atom_names(atomlist):
    AtomNames     = []
    for i in range(0,len(atomlist),2):
        if str(atomlist[i+1]) != " ":
            AtomNames.append(str(atomlist[i]) + str(atomlist[i+1]))
        else:
            AtomNames.append(str(atomlist[i]))
    return AtomNames



#----------------------------------------------------------------------#
#                          MAIN                                        #
#----------------------------------------------------------------------#
if __name__ == '__main__':
    #
    # stop if this script is called without
    # the kf-file as argument
    #
    if len(sys.argv) != 2:
        help_n_exit("\n USAGE: " + str(sys.argv[0]) + " [name of kf-file]\n")     
    kf_filename = sys.argv[1]
    #
    # check if the specified file does actually
    # exist
    #
    if not os.path.isfile(kf_filename):
        help_n_exit("\n Error: " + kf_filename + " is not a file.\n")
    #
    # Initialize plams
    #
    init()
    #
    # Initialize the PLAMS KFReader 
    #
    try:
        KFReader = KFFile(kf_filename)
    except:
        exception = sys.exc_info()[0]
        help_n_exit("\n tried to initialize KFFile: " + kf_filename + " ->  Error: %s " % exception + "\n")
    
    #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!#
    #                                                                 #
    #               GCMC - code                                       #
    #                                                                 # 
    #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!#
    #
    # get the stepnumbers
    #
    Steps = KFReader.read('General', 'Step numbers')
    Steps = filter(lambda a: a != -1, Steps)
    #
    # initial energy
    #
    Energies = KFReader.read('History', 'Energies 1')  
    InitialEnergy = Energies[15] # see KFBrowser, remember: Python starts counting at 0
    
    #
    # open outputfile
    # 
    fout = open('gcmc_convergence.out','w')
    #
    # loop through steps
    #
    for step in Steps:
        NumAtoms      = KFReader.read('History', 'Number of atoms ' + str(step))
        AtomNamesBuff = KFReader.read('History', 'Atom names ' + str(step))
        AtomNames     = clean_atom_names(AtomNamesBuff)
        Energies      = KFReader.read('History', 'Energies ' + str(step))
        #
        # calculate atom-ratio
        #
        atom_count1 = 0.0
        atom_count2 = 0.0
        for atom in AtomNames:
            if atom == "Pd":
                atom_count1 += 1
            else:
                atom_count2 += 1
        #
        # calculate relative energy
        #        
        ERel = Energies[15] - InitialEnergy - (atom_count2*(-71.04))
        print step, Energies[15], - InitialEnergy, atom_count2, ERel
        fout.write(str(step) + " " + str(atom_count2/atom_count1)+ " " + str(ERel) + "\n")
    #
    # close outputfile
    #
    fout.close()
    #   
    # Finish PLAMS
    #        
    finish()
    #
    # remove the plams working directory 
    #
    for name in glob.glob('plams.*'):
        shutil.rmtree(name, ignore_errors=True)
