#!/usr/bin/env startpython
from __future__ import print_function
#########################################################################
#                                                                       # 
#         PLAMS TEMPLATE FOR READING FROM KF-FILES                      # 
#                                                                       # 
#     More information:                                                 # 
#     PLAMS - www.scm.com/doc/Scripting/PLAMS/PLAMS.html                #
#     KF    - www.scm.com/doc/ADF/Appendices/Binary_result_files.html   #
#                                                                       #
#########################################################################


import sys
import os
import glob
import shutil
from scm.plams import *
from rdkit import Chem

#
# print help and exit
#
def help_n_exit(message):
    print message
    sys.exit(1)


#----------------------------------------------------------------------#
#                          MAIN                                        #
#----------------------------------------------------------------------#
if __name__ == '__main__':
    #
    # stop if this script is called without
    # the kf-file as argument
    #
    if len(sys.argv) != 2:
        help_n_exit("\n USAGE: " + str(sys.argv[0]) + " [name of kf-file]\n")     
    kf_filename = sys.argv[1]
    #
    # check if the specified file does actually
    # exist
    #
    if not os.path.isfile(kf_filename):
        help_n_exit("\n Error: " + kf_filename + " is not a file.\n")
    #
    # Initialize plams
    #
    init()
    #
    # Initialize the PLAMS KFReader 
    #
    try:
        KFReader = KFFile(kf_filename)
    except:
        exception = sys.exc_info()[0]
        help_n_exit("\n tried to initialize KFFile: " + kf_filename + " ->  Error: %s " % exception + "\n")
    
    #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!#
    #                                                                 #
    #               YOUR CODE HERE....                                #
    #                                                                 # 
    #!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!#
    Steps = KFReader.read('General', 'Step numbers')
    Steps = filter(lambda a: a != -1, Steps)
    for step in Steps:
        NumNeighbors = KFReader.read('History', 'ConnTab num neighb ' + str(step))
        NumDefects = 0
        for neighbors in NumNeighbors:
            if neighbors != 3:
                NumDefects += 1
        print step, " ", NumDefects
    #           
    # Finish PLAMS
    #        
    finish()
    #
    # remove the plams working directory 
    #
    for name in glob.glob('plams.*'):
        shutil.rmtree(name, ignore_errors=True)
